// tempus 2000.11g
#ifndef BLAT01_SYSTEM_CLASS
#define BLAT01_SYSTEM_CLASS
//
// (c) Copyright 1995-2001, MZA Associates Corporation
//

#include "tempus.h"

#include "TempusInitializers.h"
#include "RecallableFunctions.h"
#include "GaussianCwLaser.h"
#include "WindAtmoPath.h"
#include "BeamSteeringMirror.h"
#include "IncomingSplitter.h"
#include "DeformableMirror.h"
#include "TrackController.h"
#include "AOController.h"
#include "Target.h"
#include "TelescopeSys.h"
#include "IncomingSplitter.h"

class BLAT01 : public System
{
public:

   // Parameters
   float range;
   float tel_focus_range;
   AcsAtmSpec AtmSpec;
   int AtmSeed;
   float tmax;
   Vector<float> vplat;
   Vector<float> vtarg;
   Vector<float> vwind;
   DMModel& dmModel;
   float Dap;
   float hel_wavelength;
   float beacon_wavelength;
   float img_wavelength;
   float beacon_x;
   float beacon_y;
   int propnxy;
   float propdxy;
   int tbd_nxy;
   float tbd_dxy;
   double control_interval;
   int trk_nxy;
   float trk_dxy;
   float trk_gain;
   float bsm_timeConstant;
   float AO_gain;
   float dm_timeConstant;
   float wfs_ifov;
   bool img_on;
   int img_nxy;
   float img_dxy;
   Vector<float> img_x;
   Vector<float> img_y;

   // Subsystems
   GaussianCwLaser HEL;

   WindAtmoPath Atmosphere;

   BeamSteeringMirror BSM;

   IncomingSplitter BS;

   DeformableMirror DM;

   TrackController Tracker;

   AOController WFS;

   Target Target;

   TelescopeSys Telescope_;

   IncomingSplitter ASE;


   // Inputs

   // Outputs

private:      // Setting expressions for unsatisfied inputs/external outputs
   Output< bool >  HEL_on;

public:
   BLAT01(SystemNode *parent, char *name,
          float _range,
          float _tel_focus_range,
          AcsAtmSpec _AtmSpec,
          int _AtmSeed,
          float _tmax,
          Vector<float> _vplat,
          Vector<float> _vtarg,
          Vector<float> _vwind,
          DMModel& _dmModel,
          float _Dap,
          float _hel_wavelength,
          float _beacon_wavelength,
          float _img_wavelength,
          float _beacon_x,
          float _beacon_y,
          int _propnxy,
          float _propdxy,
          int _tbd_nxy,
          float _tbd_dxy,
          double _control_interval,
          int _trk_nxy,
          float _trk_dxy,
          float _trk_gain,
          float _bsm_timeConstant,
          float _AO_gain,
          float _dm_timeConstant,
          float _wfs_ifov,
          bool _img_on,
          int _img_nxy,
          float _img_dxy,
          Vector<float> _img_x,
          Vector<float> _img_y) : 
      System(parent, name),
      range(_range),
      tel_focus_range(_tel_focus_range),
      AtmSpec(_AtmSpec),
      AtmSeed(_AtmSeed),
      tmax(_tmax),
      vplat(_vplat),
      vtarg(_vtarg),
      vwind(_vwind),
      dmModel(_dmModel),
      Dap(_Dap),
      hel_wavelength(_hel_wavelength),
      beacon_wavelength(_beacon_wavelength),
      img_wavelength(_img_wavelength),
      beacon_x(_beacon_x),
      beacon_y(_beacon_y),
      propnxy(_propnxy),
      propdxy(_propdxy),
      tbd_nxy(_tbd_nxy),
      tbd_dxy(_tbd_dxy),
      control_interval(_control_interval),
      trk_nxy(_trk_nxy),
      trk_dxy(_trk_dxy),
      trk_gain(_trk_gain),
      bsm_timeConstant(_bsm_timeConstant),
      AO_gain(_AO_gain),
      dm_timeConstant(_dm_timeConstant),
      wfs_ifov(_wfs_ifov),
      img_on(_img_on),
      img_nxy(_img_nxy),
      img_dxy(_img_dxy),
      img_x(_img_x),
      img_y(_img_y),
      HEL(this, "HEL",
         1.0,
         hel_wavelength,
         Dap,
         0.0,
         (Dap*(6.0/7.0)),
         propnxy*propdxy,
         propnxy),
      Atmosphere(this, "Atmosphere",
         AtmSpec,
         AtmSeed,
         propnxy,
         propdxy,
         vplat,
         vtarg,
         vwind,
         tmax,
         Dap),
      BSM(this, "BSM",
         0.0,
         bsm_timeConstant,
         Vector<float>(),
         Vector<float>(),
         Vector<float>()),
      BS(this, "BS"),
      DM(this, "DM",
         dmModel,
         0.0,
         dm_timeConstant,
         Vector<float>(),
         Vector<float>(),
         Vector<float>()),
      Tracker(this, "Tracker",
         range,
         control_interval,
         trk_gain,
         Dap,
         propdxy,
         beacon_wavelength,
         trk_nxy,
         trk_dxy,
         img_wavelength,
         img_nxy,
         img_dxy,
         img_on),
      WFS(this, "WFS",
         range,
         dmModel,
         propnxy,
         propdxy,
         beacon_wavelength,
         wfs_ifov,
         control_interval,
         AO_gain),
      Target(this, "Target",
         control_interval,
         range,
         vtarg,
         hel_wavelength,
         tbd_nxy,
         tbd_dxy,
         beacon_wavelength,
         beacon_x,
         beacon_y,
         img_wavelength,
         img_x,
         img_y),
      Telescope_(this, "Telescope_",
         range,
         tel_focus_range,
         Dap,
         vplat,
         vtarg,
         propdxy,
         control_interval,
         beacon_wavelength),
      ASE(this, "ASE"),
      HEL_on(this, "HEL_on")
   {
      DM.incomingIncident <<= BSM.incomingTransmitted;
      BSM.outgoingIncident <<= DM.outgoingTransmitted;
      BS.incomingIncident <<= DM.incomingTransmitted;
      BSM.tiltCommands <<= Tracker.trackcommand;
      DM.actuatorCommands <<= WFS.dmcommands;
      Atmosphere.incomingincident <<= Target.transmitted;
      Telescope_.incomingIncident <<= Atmosphere.incomingtransmitted;
      Atmosphere.outgoingincident <<= Telescope_.outgoingTransmitted;
      BSM.incomingIncident <<= Telescope_.incomingTransmitted;
      Telescope_.outgoingIncident <<= BSM.outgoingTransmitted;
      DM.outgoingIncident <<= BS.outgoingTransmitted;
      ASE.incomingIncident <<= BS.incomingTransmitted2;
      ASE.outgoingIncident <<= HEL.transmitted;
      BS.outgoingIncident <<= ASE.outgoingTransmitted;
      Tracker.incident <<= BS.incomingTransmitted;
      WFS.incident <<= ASE.incomingTransmitted;
      Target.incident <<= Atmosphere.outgoingtransmitted;
      WFS.actuatorpositions <<= DM.measuredActuatorPositions;
      Tracker.actuatorpositions <<= BSM.measuredTilts;

      // System's Outputs and Subsystems' Inputs that
      // are not connected, but have setting expressions
      HEL.on <<= HEL_on;
   }

   // void respondToScheduledEvent (const Event & /* event */);
   // void respondToChangedInputs();
   // void respondToInputWarning (const InputBase* /* input */);
   void respondToOutputRequest (const OutputBase* output)
   {
      double t = now();
      if(*output == HEL_on)
      {
         HEL_on = (bool)(true);
      }
   }
};

#endif // BLAT01_SYSTEM_CLASS
