// tempus 2000.11g
#ifndef TRACKCONTROLLER_SYSTEM_CLASS
#define TRACKCONTROLLER_SYSTEM_CLASS
//
// (c) Copyright 1995-2001, MZA Associates Corporation
//

#include "tempus.h"


#include "Camera.h"
#include "Centroid.h"
#include "TriggeredWeightedSum.h"
#include "SquareWave.h"
#include "IncomingSplitter.h"
#include "IncomingSplitter.h"
#include "Camera.h"
#include "SimpleFieldSensor.h"
#include "BoolAnd.h"

class TrackController : public System
{
public:

   // Parameters
   float range;
   double control_interval;
   float trk_gain;
   float Dap;
   float propdxy;
   float trk_wavelength;
   int trk_nxy;
   float trk_dxy;
   float img_wavelength;
   int img_nxy;
   float img_dxy;
   bool img_on;

   // Subsystems
   Camera trkcamera;

   Centroid trkcentroid;

   TriggeredWeightedSum trkcommand;

   SquareWave zoverc;

   IncomingSplitter BS;

   IncomingSplitter BS2;

   Camera imgcamera;

   SimpleFieldSensor sfs;

   BoolAnd booland;


   // Inputs
   Input< WaveTrain > incident;
   Input< Vector<float> > actuatorpositions;

   // Outputs
   Output< Vector<float> > trackcommand;

private:      // Setting expressions for unsatisfied inputs/external outputs
   Output< double >  trkcamera_exposureInterval;
   Output< double >  trkcamera_exposureLength;
   Output< double >  trkcamera_sampleInterval;
   Output< WaveTrain >  BS_outgoingIncident;
   Output< WaveTrain >  BS2_outgoingIncident;
   Output< double >  imgcamera_exposureInterval;
   Output< double >  imgcamera_exposureLength;
   Output< double >  imgcamera_sampleInterval;
   Output< double >  sfs_exposureInterval;
   Output< double >  sfs_exposureLength;
   Output< double >  sfs_sampleInterval;
   Output< bool >  booland_b1;

public:
   TrackController(SystemNode *parent, char *name,
                   float _range,
                   double _control_interval,
                   float _trk_gain,
                   float _Dap,
                   float _propdxy,
                   float _trk_wavelength,
                   int _trk_nxy,
                   float _trk_dxy,
                   float _img_wavelength,
                   int _img_nxy,
                   float _img_dxy,
                   bool _img_on) : 
      System(parent, name),
      range(_range),
      control_interval(_control_interval),
      trk_gain(_trk_gain),
      Dap(_Dap),
      propdxy(_propdxy),
      trk_wavelength(_trk_wavelength),
      trk_nxy(_trk_nxy),
      trk_dxy(_trk_dxy),
      img_wavelength(_img_wavelength),
      img_nxy(_img_nxy),
      img_dxy(_img_dxy),
      img_on(_img_on),
      trkcamera(this, "trkcamera",
         1.0,
         trk_wavelength,
         trk_wavelength,
         Dap / propdxy,
         propdxy,
         trk_nxy,
         trk_dxy,
         0.0),
      trkcentroid(this, "trkcentroid"),
      trkcommand(this, "trkcommand",
         -trk_gain,
         1.0),
      zoverc(this, "zoverc",
         (range/speedOfLight),
         1.0,
         1.0e+06,
         1.0e+06),
      BS(this, "BS"),
      BS2(this, "BS2"),
      imgcamera(this, "imgcamera",
         1.0,
         img_wavelength,
         img_wavelength,
         Dap / propdxy,
         propdxy,
         img_nxy,
         img_dxy,
         0.0),
      sfs(this, "sfs",
         trk_wavelength,
         Dap /propdxy,
         propdxy),
      booland(this, "booland",
         false),
      incident(this, "incident"),
      actuatorpositions(this, "actuatorpositions"),
      trackcommand(this, "trackcommand"),
      trkcamera_exposureInterval(this, "trkcamera_exposureInterval"),
      trkcamera_exposureLength(this, "trkcamera_exposureLength"),
      trkcamera_sampleInterval(this, "trkcamera_sampleInterval"),
      BS_outgoingIncident(this, "BS_outgoingIncident"),
      BS2_outgoingIncident(this, "BS2_outgoingIncident"),
      imgcamera_exposureInterval(this, "imgcamera_exposureInterval"),
      imgcamera_exposureLength(this, "imgcamera_exposureLength"),
      imgcamera_sampleInterval(this, "imgcamera_sampleInterval"),
      sfs_exposureInterval(this, "sfs_exposureInterval"),
      sfs_exposureLength(this, "sfs_exposureLength"),
      sfs_sampleInterval(this, "sfs_sampleInterval"),
      booland_b1(this, "booland_b1")
   {
      trkcentroid.grid <<= trkcamera.fpaImage;
      trkcommand.summand1 <<= trkcentroid.v;
      trkcamera.on <<= zoverc.boolOutputSignal;
      trackcommand <<= trkcommand.sum;
      trkcommand.summand2 <<= actuatorpositions;
      BS.incomingIncident <<= incident;
      BS2.incomingIncident <<= BS.incomingTransmitted;
      trkcamera.incident <<= BS2.incomingTransmitted;
      imgcamera.incident <<= BS.incomingTransmitted2;
      sfs.incident <<= BS2.incomingTransmitted2;
      booland.b0 <<= zoverc.boolOutputSignal;
      imgcamera.on <<= booland.b;
      sfs.on <<= zoverc.boolOutputSignal;

      // System's Outputs and Subsystems' Inputs that
      // are not connected, but have setting expressions
      trkcamera.exposureInterval <<= trkcamera_exposureInterval;
      trkcamera.exposureLength <<= trkcamera_exposureLength;
      trkcamera.sampleInterval <<= trkcamera_sampleInterval;
      BS.outgoingIncident <<= BS_outgoingIncident;
      BS2.outgoingIncident <<= BS2_outgoingIncident;
      imgcamera.exposureInterval <<= imgcamera_exposureInterval;
      imgcamera.exposureLength <<= imgcamera_exposureLength;
      imgcamera.sampleInterval <<= imgcamera_sampleInterval;
      sfs.exposureInterval <<= sfs_exposureInterval;
      sfs.exposureLength <<= sfs_exposureLength;
      sfs.sampleInterval <<= sfs_sampleInterval;
      booland.b1 <<= booland_b1;
   }

   // void respondToScheduledEvent (const Event & /* event */);
   // void respondToChangedInputs();
   // void respondToInputWarning (const InputBase* /* input */);
   void respondToOutputRequest (const OutputBase* output)
   {
      double t = now();
      if(*output == trkcamera_exposureInterval)
      {
         trkcamera_exposureInterval = (double)(control_interval);
      }
      else if(*output == trkcamera_exposureLength)
      {
         trkcamera_exposureLength = (double)(1.0e-6);
      }
      else if(*output == trkcamera_sampleInterval)
      {
         trkcamera_sampleInterval = (double)(-1.0);
      }
      else if(*output == BS_outgoingIncident)
      {
         BS_outgoingIncident = (WaveTrain)(WaveTrain());
      }
      else if(*output == BS2_outgoingIncident)
      {
         BS2_outgoingIncident = (WaveTrain)(WaveTrain());
      }
      else if(*output == imgcamera_exposureInterval)
      {
         imgcamera_exposureInterval = (double)(control_interval);
      }
      else if(*output == imgcamera_exposureLength)
      {
         imgcamera_exposureLength = (double)(1.0e-06);
      }
      else if(*output == imgcamera_sampleInterval)
      {
         imgcamera_sampleInterval = (double)(-1.0);
      }
      else if(*output == sfs_exposureInterval)
      {
         sfs_exposureInterval = (double)(control_interval);
      }
      else if(*output == sfs_exposureLength)
      {
         sfs_exposureLength = (double)(1.0e-6);
      }
      else if(*output == sfs_sampleInterval)
      {
         sfs_sampleInterval = (double)(-1.0);
      }
      else if(*output == booland_b1)
      {
         booland_b1 = (bool)(img_on);
      }
   }
};

#endif // TRACKCONTROLLER_SYSTEM_CLASS
